@@ -4,7 +4,7 @@ written by Michael Stroeder <michael@stroeder.com>
 
 See http://python-ldap.sourceforge.net for details.
 
-\$Id: ldapobject.py,v 1.83 2004/11/09 01:13:41 stroeder Exp $
+\$Id: ldapobject.py,v 1.91 2005/09/22 04:26:12 stroeder Exp $
 
 Compability:
 - Tested with Python 2.0+ but should work with Python 1.5.x
@@ -37,7 +37,7 @@ if __debug__:
 import sys,time,_ldap,ldap,ldap.functions
 
 from ldap.schema import SCHEMA_ATTRS
-
+from ldap.controls import LDAPControl,DecodeControlTuples,EncodeControlTuples
 from ldap import LDAPError
 
 class SimpleLDAPObject:
@@ -96,7 +96,7 @@ class SimpleLDAPObject:
         self._ldap_object_lock.release()
     except LDAPError,e:
       if __debug__ and self._trace_level>=2:
-        self._trace_file.write('=> LDAPError: %s\n' % (str(e)))
+        self._trace_file.write('=> LDAPError - %s: %s\n' % (e.__class__.__name__,str(e)))
       raise
     if __debug__ and self._trace_level>=2:
       if result!=None and result!=(None,None):
@@ -129,7 +129,7 @@ class SimpleLDAPObject:
         can expect that the result of an abandoned operation will not be
         returned from a future call to result().
     """
-    return self._ldap_call(self._l.abandon_ext,msgid,serverctrls,clientctrls)
+    return self._ldap_call(self._l.abandon_ext,msgid,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
 
   def abandon(self,msgid):
     return self.abandon_ext(msgid,None,None)
@@ -142,7 +142,7 @@ class SimpleLDAPObject:
         The parameter modlist is similar to the one passed to modify(),
         except that no operation integer need be included in the tuples.
     """
-    return self._ldap_call(self._l.add_ext,dn,modlist,serverctrls,clientctrls)
+    return self._ldap_call(self._l.add_ext,dn,modlist,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
 
   def add_ext_s(self,dn,modlist,serverctrls=None,clientctrls=None):
     msgid = self.add_ext(dn,modlist,serverctrls,clientctrls)
@@ -166,7 +166,7 @@ class SimpleLDAPObject:
     """
     simple_bind([who='' [,cred='']]) -> int
     """
-    return self._ldap_call(self._l.simple_bind,who,cred,serverctrls,clientctrls)
+    return self._ldap_call(self._l.simple_bind,who,cred,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
 
   def simple_bind_s(self,who='',cred='',serverctrls=None,clientctrls=None):
     """
@@ -193,7 +193,7 @@ class SimpleLDAPObject:
     """
     sasl_interactive_bind_s(who, auth) -> None
     """
-    return self._ldap_call(self._l.sasl_interactive_bind_s,who,auth,serverctrls,clientctrls,sasl_flags)
+    return self._ldap_call(self._l.sasl_interactive_bind_s,who,auth,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls),sasl_flags)
 
   def compare_ext(self,dn,attr,value,serverctrls=None,clientctrls=None):
     """
@@ -213,7 +213,7 @@ class SimpleLDAPObject:
         A design bug in the library prevents value from containing
         nul characters.
     """
-    return self._ldap_call(self._l.compare_ext,dn,attr,value,serverctrls,clientctrls)
+    return self._ldap_call(self._l.compare_ext,dn,attr,value,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
 
   def compare_ext_s(self,dn,attr,value,serverctrls=None,clientctrls=None):
     msgid = self.compare_ext(dn,attr,value,serverctrls,clientctrls)
@@ -241,7 +241,7 @@ class SimpleLDAPObject:
         form returns the message id of the initiated request, and the
         result can be obtained from a subsequent call to result().
     """
-    return self._ldap_call(self._l.delete_ext,dn,serverctrls,clientctrls)
+    return self._ldap_call(self._l.delete_ext,dn,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
 
   def delete_ext_s(self,dn,serverctrls=None,clientctrls=None):
     msgid = self.delete_ext(dn,serverctrls,clientctrls)
@@ -259,13 +259,20 @@ class SimpleLDAPObject:
     manage_dsa_it() -> None
     Enable or disable manageDSAit mode (see draft-zeilenga-ldap-namedref)
     """
-    self._ldap_call(self._l.manage_dsa_it,enable,critical)
+    all_server_ctrls = [
+      c
+      for c in self.get_option(ldap.OPT_SERVER_CONTROLS) or []
+      if c.controlType!='2.16.840.1.113730.3.4.2'
+    ]    
+    if enable:
+      all_server_ctrls.append(LDAPControl('2.16.840.1.113730.3.4.2',critical,None))
+    self.set_option(ldap.OPT_SERVER_CONTROLS,all_server_ctrls)
   
   def modify_ext(self,dn,modlist,serverctrls=None,clientctrls=None):
     """
     modify_ext(dn, modlist[,serverctrls=None[,clientctrls=None]]) -> int
     """
-    return self._ldap_call(self._l.modify_ext,dn,modlist,serverctrls,clientctrls)
+    return self._ldap_call(self._l.modify_ext,dn,modlist,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
 
   def modify_ext_s(self,dn,modlist,serverctrls=None,clientctrls=None):
     msgid = self.modify_ext(dn,modlist,serverctrls,clientctrls)
@@ -318,7 +325,7 @@ class SimpleLDAPObject:
     self.rename_s(dn,newrdn,None,delold)
 
   def passwd(self,user,oldpw,newpw,serverctrls=None,clientctrls=None):
-    return self._ldap_call(self._l.passwd,user,oldpw,newpw,serverctrls,clientctrls)
+    return self._ldap_call(self._l.passwd,user,oldpw,newpw,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
 
   def passwd_s(self,user,oldpw,newpw,serverctrls=None,clientctrls=None):
     msgid = self.passwd(user,oldpw,newpw,serverctrls,clientctrls)
@@ -339,7 +346,7 @@ class SimpleLDAPObject:
         This actually corresponds to the rename* routines in the
         LDAP-EXT C API library.
     """
-    return self._ldap_call(self._l.rename,dn,newrdn,newsuperior,delold,serverctrls,clientctrls)
+    return self._ldap_call(self._l.rename,dn,newrdn,newsuperior,delold,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
 
   def rename_s(self,dn,newrdn,newsuperior=None,delold=1):
     msgid = self.rename(dn,newrdn,newsuperior,delold)
@@ -398,11 +405,17 @@ class SimpleLDAPObject:
     """
     res_type,res_data,res_msgid = self.result2(msgid,all,timeout)
     return res_type,res_data
- 
+
   def result2(self,msgid=_ldap.RES_ANY,all=1,timeout=None):
+    res_type, res_data, res_msgid, srv_ctrls = self.result3(msgid,all,timeout)
+    return res_type, res_data, res_msgid
+ 
+  def result3(self,msgid=_ldap.RES_ANY,all=1,timeout=None):
     if timeout is None:
       timeout = self.timeout
-    return self._ldap_call(self._l.result2,msgid,all,timeout)
+    rtype, rdata, rmsgid, serverctrls = self._ldap_call(self._l.result3,msgid,all,timeout)
+    decoded_serverctrls = DecodeControlTuples(serverctrls)
+    return rtype, rdata, rmsgid, decoded_serverctrls
  
   def search_ext(self,base,scope,filterstr='(objectClass=*)',attrlist=None,attrsonly=0,serverctrls=None,clientctrls=None,timeout=-1,sizelimit=0):
     """
@@ -453,22 +466,23 @@ class SimpleLDAPObject:
       self._l.search_ext,
       base,scope,filterstr,
       attrlist,attrsonly,
-      serverctrls,clientctrls,
-      timeout,sizelimit
+      EncodeControlTuples(serverctrls),
+      EncodeControlTuples(clientctrls),
+      timeout,sizelimit,
     )
 
   def search_ext_s(self,base,scope,filterstr='(objectClass=*)',attrlist=None,attrsonly=0,serverctrls=None,clientctrls=None,timeout=-1,sizelimit=0):
-    msgid = self._ldap_call(self._l.search_ext,base,scope,filterstr,attrlist,attrsonly,serverctrls,clientctrls,timeout,sizelimit)
+    msgid = self.search_ext(base,scope,filterstr,attrlist,attrsonly,serverctrls,clientctrls,timeout,sizelimit)
     return self.result(msgid,all=1,timeout=timeout)[1]
 
   def search(self,base,scope,filterstr='(objectClass=*)',attrlist=None,attrsonly=0):
-    return self.search_ext(base,scope,filterstr,attrlist,attrsonly)
+    return self.search_ext(base,scope,filterstr,attrlist,attrsonly,None,None)
 
   def search_s(self,base,scope,filterstr='(objectClass=*)',attrlist=None,attrsonly=0):
-    return self.search_ext_s(base,scope,filterstr,attrlist,attrsonly,timeout=self.timeout)
+    return self.search_ext_s(base,scope,filterstr,attrlist,attrsonly,None,None,timeout=self.timeout)
 
   def search_st(self,base,scope,filterstr='(objectClass=*)',attrlist=None,attrsonly=0,timeout=-1):
-    return self.search_ext_s(base,scope,filterstr,attrlist,attrsonly,timeout)
+    return self.search_ext_s(base,scope,filterstr,attrlist,attrsonly,None,None,timeout)
 
   def set_cache_options(self,*args,**kwargs):
     """
@@ -523,7 +537,7 @@ class SimpleLDAPObject:
         The unbind and unbind_s methods are identical, and are
         synchronous in nature
     """
-    return self._ldap_call(self._l.unbind_ext,serverctrls,clientctrls)
+    return self._ldap_call(self._l.unbind_ext,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
 
   def unbind_ext_s(self,serverctrls=None,clientctrls=None):
     msgid = self.unbind_ext(serverctrls,clientctrls)
@@ -542,9 +556,14 @@ class SimpleLDAPObject:
     return self._ldap_call(self._l.whoami_s,serverctrls,clientctrls)
 
   def get_option(self,option):
-    return self._ldap_call(self._l.get_option,option)
+    result = self._ldap_call(self._l.get_option,option)
+    if option==ldap.OPT_SERVER_CONTROLS or option==ldap.OPT_CLIENT_CONTROLS:
+      result = DecodeControlTuples(result)
+    return result
 
   def set_option(self,option,invalue):
+    if option==ldap.OPT_SERVER_CONTROLS or option==ldap.OPT_CLIENT_CONTROLS:
+      invalue = EncodeControlTuples(invalue)
     self._ldap_call(self._l.set_option,option,invalue)
 
   def search_subschemasubentry_s(self,dn=''):
@@ -559,7 +578,7 @@ class SimpleLDAPObject:
       r = self.search_s(
         dn,ldap.SCOPE_BASE,'(objectClass=*)',['subschemaSubentry']
       )
-    except (ldap.NO_SUCH_OBJECT,ldap.NO_SUCH_ATTRIBUTE):
+    except (ldap.NO_SUCH_OBJECT,ldap.NO_SUCH_ATTRIBUTE,ldap.INSUFFICIENT_ACCESS):
       r = []
     except ldap.UNDEFINED_TYPE:
       return None
